"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var http = require("http");
function unwrapUndefined(value) {
    return value;
}
var PromiseHttpServer = (function () {
    function PromiseHttpServer() {
        var _this = this;
        this.resolveAndRejectQueue = [];
        this.reqResOrErrorQueue = [];
        this.server = http.createServer(function (req, res) {
            if (_this.resolveAndRejectQueue.length === 0) {
                _this.reqResOrErrorQueue.push({
                    kind: "REQ_RES",
                    req: req,
                    res: res
                });
            }
            else {
                var resolveAndReject = unwrapUndefined(_this.resolveAndRejectQueue.shift());
                resolveAndReject.resolve({
                    req: req,
                    res: res
                });
            }
        }).on('error', function (err) {
            if (_this.resolveAndRejectQueue.length === 0) {
                _this.reqResOrErrorQueue.push({
                    kind: "ERROR",
                    error: err
                });
            }
            else {
                var resolveAndReject = unwrapUndefined(_this.resolveAndRejectQueue.shift());
                resolveAndReject.reject(err);
            }
        });
    }
    PromiseHttpServer.prototype.accept = function () {
        var _this = this;
        return new Promise((function (resolve, reject) {
            if (_this.reqResOrErrorQueue.length === 0) {
                _this.resolveAndRejectQueue.push({
                    resolve: resolve,
                    reject: reject
                });
            }
            else {
                var reqResOrError = unwrapUndefined(_this.reqResOrErrorQueue.shift());
                switch (reqResOrError.kind) {
                    case "REQ_RES":
                        resolve({
                            req: reqResOrError.req,
                            res: reqResOrError.res
                        });
                        break;
                    case "ERROR":
                        reject(reqResOrError.error);
                        break;
                }
            }
        }));
    };
    PromiseHttpServer.prototype.listen = function (port) {
        var _this = this;
        return new Promise(function (resolve) {
            _this.server.listen(port, resolve);
        });
    };
    PromiseHttpServer.prototype.close = function () {
        var _this = this;
        return new Promise(function (resolve) {
            _this.server.close(resolve);
        });
    };
    return PromiseHttpServer;
}());
exports.PromiseHttpServer = PromiseHttpServer;
//# sourceMappingURL=index.js.map